/**
 * E-FancyLauncher to FancyLauncher converter
 * ease the move to the new version that can
 * be run without Enlightenment.
 *
 * author: Alexandre David
 * http://www.docs.uu.se/~adavid
 */

/* This is to generate the default
 * configuration, i.e. default_config.h
 * If you want to regenerate it, change
 * this and run the new compiled version
 * with your E-FancyLauncher configuration
 * file as argument.
 */

#include<stdio.h>
#include<stdlib.h>
#include "debug.h"

typedef struct {
  char key[32];
  char val[1024];
} ConfEntry;

#include"default_config.h"

debug( int mem_alloc=0 );
debug( int printflag=1 );

int nconf=0;
ConfEntry *config=NULL;
int generate_default=0;

void initConfig(const char *filename) {
  FILE *f=(filename ? fopen(filename,"r") : NULL);
  if (f) {
    char buf[1064];
    printf("Converting %s...\n",filename);
    while(fgets(buf,sizeof(buf),f)) {
      if (buf[0]!='#' && buf[0]!='\n') {
	int len;
	nconf++;
	config=(ConfEntry*)realloc(config,nconf*sizeof(ConfEntry));
	if (!config) {
	  fprintf(stderr,"realloc error, aborted.\n");
	  exit(1);
	}
	/* dangerous, may buffer overflow */
	sscanf(buf,"%s %[^\n]\n",
	       config[nconf-1].key,
	       config[nconf-1].val);
	
	len=strlen(config[nconf-1].val);
	while(len>0 && (config[nconf-1].val[len-1]==' ' || config[nconf-1].val[len-1]=='\t'))
	  len--; /* strip */
	config[nconf-1].val[len]='\0';
      }
    }
    fclose(f);
  } else {
    if (!generate_default) {
      fprintf(stderr,"Could not open %s, aborted.\n",filename);
      exit(1);
    } else {
      printf("Taking default configuration...\n");
      config=default_config;
      nconf=sizeof(default_config)/sizeof(ConfEntry);
    }
  }
}

const char *getVal(const char *key,const char *defaultVal) {
  int i;
  for(i=0;i<nconf;i++)
    if (!strcmp(config[i].key,key))
      return config[i].val;
  return defaultVal;
}

int getInt(char *key,char *defaultVal,int min) {
  const char *s=getVal(key,defaultVal);
  int result=(*s == '*' ? 16*atoi(s+1) : atoi(s));
  if (result<min) return min;
  return result;
}

void convertTo(const int fileID) {
  char buf[1024];
  FILE *f;
  /* fix me: check better the following */
  if (fileID>1) {
    sprintf(buf,"%s/.FancyLauncher/config.%d",getenv("HOME"),fileID);
    fprintf(stderr,
	    "You should merge config.%d with config to get only one configuration file\n",
	    fileID);
  } else
    sprintf(buf,"%s/.FancyLauncher/config",getenv("HOME"));
  f=fopen(buf,"r");
  if (f) {
    fclose(f);
    fprintf(stderr,"File %s exists, trying config.%d\n",buf,fileID+1);
    convertTo(fileID+1);
  } else {
    f=fopen(buf,"w");
    if (f) {
      int cols=getInt("COLUMNS","2",1);
      int rows=getInt("ROWS","6",1);
      int icon_w=getInt("EPPLET_WIDTH","*3",1)/cols;
      int icon_h=getInt("EPPLET_HEIGHT","*9",1)/rows;
      int border=getInt("ICON_BORDER","0",0);
      int i,p,y,n=cols*rows;
      const char *sa,*s;
      
      i=(((getInt("TRANSP_RAISE","0",0))|(getInt("RAISE","1",0)))!=0);
      printf("Creating %s...\n",buf);
      fprintf(f,
	      "# Common options\n"
	      "common {\n"
	      "\n"
	      "# Button dimension, can be redefined for each button\n"
	      "  width %d\n"
	      "  height %d\n"
	      "\n"
	      "# Flags can be: 0, 1, true, false, on, off\n"
	      "  blend_icon 200\n"
	      "  blend_background 128\n"
	      "  background %s\n"
	      "  tooltips %d\n"
	      "  sticky 1\n"
	      "\n"
	      "  transient %d\n"
	      "  raised %d\n"
	      "  etched %d\n"
	      "\n"
	      "# Lighter and darker effect in percent\n"
	      "  lighter 30\n"
	      "  darker 20\n"
	      "\n"
	      "# This is for wheel mice\n"
	      "  action 4 previous_page\n"
	      "  action 5 next_page\n"
	      "  tooltip 4 Previous page\n"
	      "  tooltip 5 Next page\n"
	      "\n"
	      "# Where to look for icons\n"
	      "  iconpath /usr/share/pixmaps\n"
	      "  iconpath /usr/share/icons/large\n"
	      "  iconpath /usr/share/icons\n"
	      "\n"
	      "# Key bindings, see /usr/include/X11/keysymdef.h for the keys (without XK_)\n"
	      "  bind Escape exit\n"
	      "  bind F1 exec netscape /usr/local/FancyLauncher/help.html&\n"
	      "  bind F2 configure\n"
	      "  bind a about\n"
	      "  bind 1 page 1\n"
	      "  bind 2 page 2\n"
	      "  bind 3 page 3\n"
	      "\n"
	      "# Default icon\n"
	      "  icon xterm.xpm\n"
	      "\n"
	      "# Common for page 1, start first row\n"
	      "  page 1\n"
	      "}\n",
	      icon_w-2*border,/* width */
	      icon_h-2*border,/* height */
	      getInt("TRANSPARENT","1",0)?"transparent\n":"0x999999\n", /* background */
	      (getInt("TOOLTIPS","1",0)==1),/* tooltips */
	      getInt("RAISE","1",0),
	      i,!i/* border */
	      );
      
      
      for(p=0;p<3;p++) {
	y=0;
	/* common page header */
	if (p>0)
	  fprintf(f,"\ncommon { page %d }\n",p+1);
	
	for(i=0;i<n;i++) {
	  /* button entry */
	  fprintf(f,"\nbutton {\n");
	  
	  /* x */
	  if (i%cols==0) {/* reset x */
	    if (cols>1 || (i==0  && p==0))
	      fprintf(f,"  x %d\n",border);
	  } else if (border) /* explicit */
	    fprintf(f,"  x %d\n",(i%cols)*icon_w+border);
	  else /* increment */
	    fprintf(f,"  x+ width\n");
	  
	  /* y */
	  if (i%cols==0) {
	    if (i==0) {/* reset */
	      if (rows>1 || p==0)
		fprintf(f,"  y %d\n",border);
	    } else if (border) /* explicit */
	      fprintf(f,"  y %d\n",y+border);
	    else /* increment */
	      fprintf(f,"  y+ height\n");
	    y+=icon_h;
	  }
	  
	  sprintf(buf,"PAGE%d_ICON%d",p,i);
	  s=getVal(buf,NULL);
	  if (s)
	    fprintf(f,"  icon %s\n",s);
	  
	  sprintf(buf,"PAGE%d_PROG%d",p,i);
	  sa=getVal(buf,NULL);
	  if (sa) {
	    fprintf(f,"  action 1 ");
	    /* translate commands */
	    if (!strcmp(sa,"_SWITCH"))
	      fprintf(f,
		      "page 1\n"
		      "  action 2 page 2\n"
		      "  action 3 page 3\n");
	    else if (!strcmp(sa,"_CONFIG"))
	      fprintf(f,"configure\n");
	    else if (!strcmp(sa,"_QUIT"))
	      fprintf(f,"exit\n");
	    else if (!strcmp(sa,"_INFO"))
	      fprintf(f,"exec netscape /usr/local/FancyLauncher/help.html &\n");
	    else {
	      /* execute command in background or not */
	      if (*sa=='!')
		fprintf(f,"wait %s\n",sa+1);
	      else
		fprintf(f,"exec %s\n",sa);
	    }
	  }
	  
	  sprintf(buf,"PAGE%d_TOOLTIP%d",p,i);
	  s=getVal(buf,NULL);
	  if (s) {
	    fprintf(f,"  tooltip 1 %s",s);
	    /* several tooltips if switch command */
	    if (sa && !strcmp(sa,"_SWITCH"))
	      fprintf(f,"%s%s",
		      ( s[strlen(s)-1]==' ' ? "1\n" : " 1\n" ),
		      "  tooltip 2 Switch page 2\n"
		      "  tooltip 3 Switch page 3");
	    fprintf(f,"\n");
	  }
	  fprintf(f,"}\n");
	}
      }
      fflush(f);
      fclose(f);
    } else {
      fprintf(stderr,"Could not create %s, aborted.\n",buf);
      exit(1);
    }
  }
}

void saveDefaultConfig() {
  FILE *f=fopen("default_config.h","w");
  if (f) {
    int i;
    printf("Creating default_config.h\n");
    fprintf(f,
	    "/* Generated default config file */\n\n"
	    "ConfEntry default_config[%d]={\n",nconf);
    for(i=0;i<nconf;i++) {
      int j,n=strlen(config[i].val);
      fprintf(f," { \"%s\" , \"",config[i].key);
      while(n>0 && (config[i].val[n-1]==' ' || config[i].val[n-1]=='\t'))
	n--; /* strip */
      for(j=0;j<n;j++)
	if (config[i].val[j]=='\"')
	  fprintf(f,"\\\"");
	else
	  fprintf(f,"%c",config[i].val[j]);
      fprintf(f,"\" }");
      if (i<nconf-1)
	fprintf(f,",\n");
      else
	fprintf(f,"\n};\n");
    }
    fclose(f);
  } else {
    fprintf(stderr,"Could not create default_config.h, aborted.\n");
    exit(1);
  }
}

int main(int argc,char *argv[]) {
  int i;
  if (argc<2) {
    fprintf(stderr,
	    "Usage:\t%s generate config_files : generate FancyLauncher config files from given E-FancyLauncher config files\n"
	    "\t%s config_file : translate a E-FancyLauncher.cfg file to default_config.h\n",
	    argv[0],argv[0]);
    exit(1);
  }
  generate_default=!strcmp(argv[1],"generate");
  if (!generate_default) {
    initConfig(argv[1]);
    saveDefaultConfig();
  } else {
    if (argc<3) {
      argc=3; /* OK because argv[2]==NULL */
      assert(argv[2]==NULL,"Bad arguments");
    }
    for(i=2;i<argc;i++) {
      initConfig(argv[i]);
      convertTo(i-1);
    }
  }
  return 0;
}
