#include "debug.h"
#include "utils.h"
#include "gadget.h"
#include "clock.h"
#include "pixmaps.h"
#include "Xcore.h"
#include "tooltips.h"


void saveClockGadget(FILE* out,Gadget *gad) {
  ClockData *ckdata=(ClockData*)PERIODIC(gad).data;

  fprintf(out,"\ndigital_clock {\n");
  if (ckdata) {
    fprintf(out,
	    "  resized_width %d\n"
	    "  resized_height %d\n",
	    PERIODIC(gad).win_width,
	    PERIODIC(gad).win_height);

    if (ckdata->label) {
      int n=ckdata->n;
      ClockLabel *label=ckdata->label;
      int i;
      for(i=0;i<n;i++) {
	int j;
	fprintf(out,"  label {\n");
	for(j=0;j<8;j++)
	  fprintf(out,"    %s %d\n",labelnames[j],label[i].color[j]);
	if (label[i].format)
	  fprintf(out,"    format %s\n",label[i].format);
	if (label[i].font)
	  fprintf(out,"    font %c %s\n",
		  label[i].font->name[0],
		  label[i].font->name+1);
	fprintf(out,"  }\n");
      }
    }
  }
}

void enteredClockGadget(Gadget *gad) {
  assert(gad && IS_PERIODIC(gad),"Wrong argument");
  debugPrint((GREEN"enteredClockGadget"BLACK"\n"));
  ANY(gad).opt[I_PAGE]=1;
}


void leftClockGadget(Gadget *gad) {
  assert(gad && IS_PERIODIC(gad),"Wrong argument");
  debugPrint((GREEN"leftClockGadget"BLACK"\n"));
  ANY(gad).opt[I_PAGE]=0;
}


double updateClockGadget(Gadget *gad,int *nfd,fd_set *fdset,int force) {
  debugPrint((GREEN"updateClockGadget"BLACK"\n"));

  if (!PERIODIC(gad).im_orig) {
    trace( PERIODIC(gad).im_orig=
	   IS_PIXMAP(gad->bg) ?
	   getImage(im_data,CHARS(GET_PIXMAP(gad->bg)),
		    PERIODIC(gad).win_width,
		    PERIODIC(gad).win_height) : NULL );
    if (PERIODIC(gad).im_orig==NULL) {
      if (IS_TRANSPARENT(gad->bg)) {
	ImlibColor imcol={1,1,1};
	trace( PERIODIC(gad).im_orig=getSolidImage(gad->width,gad->height,
						   im_data,0x010101));
	Imlib_set_image_shape(im_data,PERIODIC(gad).im_orig,&imcol);
      } else {
	trace( PERIODIC(gad).im_orig=getSolidImage(gad->width,gad->height,
						   im_data,
						   IS_COLOR(gad->bg)
						   ? GET_COLOR(gad->bg)
						   : 0x808080 ) );
      }
    }
  }

  if (PERIODIC(gad).im_orig) {
    if (!PERIODIC(gad).im_normal)
      trace(PERIODIC(gad).im_normal=Imlib_clone_image(im_data,PERIODIC(gad).im_orig));
    else {
      trace(memcpy(PERIODIC(gad).im_normal->rgb_data,
		   PERIODIC(gad).im_orig->rgb_data,
		   3*PERIODIC(gad).win_width*PERIODIC(gad).win_height));
      Imlib_changed_image(im_data,PERIODIC(gad).im_normal);
    }
  }

  if (!PERIODIC(gad).im_orig || !PERIODIC(gad).im_normal) {
    fprintf(stderr,"Could not make the image, aborting clock update!\n");
    return PERIODIC(gad).update_period;
  }

  if (PERIODIC(gad).data) {
    ClockData *ckdata=(ClockData*)PERIODIC(gad).data;
    int i;
    struct tm *timem;
    time_t timet;
    char buffer[256];

    timet=time(NULL);
    timem=localtime(&timet);

    assert((ckdata->n==0)==(ckdata->label==NULL),"Inconsistent data");
    for(i=0;i<ckdata->n;i++) {
      if (ckdata->label[i].format) {
	strftime(buffer,sizeof(buffer),ckdata->label[i].format,timem);
	trace( printStringImage(ckdata->label[i].color[6],/* x */
				ckdata->label[i].color[7],/* y */
				PERIODIC(gad).im_normal,
				buffer,
				ckdata->label[i].font,
				ckdata->label[i].color) );
	debugPrint(("Label %d,%d: %s=%s\n",
		    ckdata->label[i].color[6],
		    ckdata->label[i].color[7],
		    ckdata->label[i].format,
		    buffer));
      }
    }

    Imlib_apply_image(im_data,PERIODIC(gad).im_normal,gad->win);
    XFlush(display);

    if (tooltip_win && ANY(gad).opt[I_PAGE]) {
      /*trace(show_tooltip(gad,NULL,1));*/
      return 1.0; /* cheat */
    }

    return PERIODIC(gad).update_period;
  } else
    return 1.0;
}


void freeClockGadget(Gadget *gad) {
  ClockData *ckdata=(ClockData*)PERIODIC(gad).data;
  debugPrint(("freeClockGadget\n"));
  if (ckdata) {
    ClockLabel *label=ckdata->label;
    int n=ckdata->n;
    int i;
    for(i=0;i<n;i++)
      FREE(label[i].format);
    FREE(label);
    disalloc(ckdata);
  }
}


char *tooltipClockGadget(Gadget *gad) {
  time_t timet;

  debugPrint(("tooltipClockGadget\n"));

  timet=time(NULL);

  return ctime(&timet);
}
