/* main.c
 *
 * Part of FancyLauncher
 * author: Alexandre David
 * e-mail: adavid@docs.uu.se
 * license: GPL
 */

#include "debug.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <X11/Xlib.h>
#include <X11/Xresource.h>
#include <X11/Xutil.h>
#include "utils.h"
#include "gadget.h"
#include "load_save.h"
#include "Xcore.h"
#include "pixmaps.h"

debug( int mem_alloc=0 );

int main_argc;
char **main_argv;


static void handle_child() {
  waitpid(-1,NULL,WNOHANG);
}

static void checkUniqueInstance() {
  /* TODO */
}

static void save_before_exit() {
  saveButtons();
  exit(0);
}

static void ooups(int mamamia) {
  fprintf(stderr,"Oops, segmentation fault :(\n");
  exit(1);
}

static void clean_up() {
  free_gadgets();
  if (display)
    XCloseDisplay(display);
}

static void initSig() {
  struct sigaction    sa;

  sa.sa_handler = handle_child;
  sa.sa_flags = SA_RESTART;
  sigemptyset(&sa.sa_mask);
  sigaction(SIGCHLD, &sa, (struct sigaction *)0);

  atexit(clean_up);
  sa.sa_handler = save_before_exit;
  sigaction(SIGINT, &sa, (struct sigaction *)0);
  sigaction(SIGTERM,&sa, (struct sigaction *)0);
  sa.sa_handler = ooups;
  sigaction(SIGSEGV,&sa, (struct sigaction *)0);
}

int main(int argc,char *argv[]) {
  main_argc=argc;
  main_argv=argv;
  checkUniqueInstance();
  trace( initSig() );
  initDebug();

  display = XOpenDisplay(NULL);
  if (!display) {
    fprintf(stderr,"Cannot open display!\n");
    exit(1);
  }
  wmDeleteWindow = XInternAtom(display, "WM_DELETE_WINDOW", False);
  xid_context=XUniqueContext();
  im_data = Imlib_init(display);

  trace( initConfigFile(argv[1]) ); /* if arg => file ; else null */
  trace( configParse() );
  trace( saveButtons() ); /* at least for the password stuff */
  trace( showButtons(current_page));
  trace( showExtGadgets() );
  trace( X_loop() );
  return 0;
}




