/* pixmaps.h
 *
 * Part of FancyLauncher
 * author: Alexandre David
 * e-mail: adavid@docs.uu.se
 * license: GPL
 */

#ifndef PIXMAPS_H
#define PIXMAPS_H

#include <Imlib.h>

extern Display *display;
extern ImlibData *im_data;


/* apply effect lighter/darker on a button
 * skip 2 pixel border
 * width,height: wished dimension to modify
 * x: effect: 0-511 ; 0-255=darker ; 256-511=lighter
 */
void applyEffect(ImlibImage *im,int width,int height,int x,int border);


#define RAISE_BORDER 0xff,0xff
#define LOWER_BORDER 0,0
#define ETCH_BORDER  0,0xff

/* paint the border as raised,lowered or etched,
 * use as e.g. paintBorder(im,width,height,RAISE_BORDER)
 */
void paintBorder(ImlibImage *im,int width,int height,
		 char effect1,char effect2);


/* return an image filled with a color */
ImlibImage* getSolidImage(int width,int height,
			  ImlibData *id,int rgbpix);


/* take a snapshot of the screen ; or the root
 */
ImlibImage* snapshotImage(ImlibData* id,
			  int x,int y,int width,int height,
			  Window win);


/* search for an image in the configured paths and scale it
 */
ImlibImage* getImage(ImlibData *id,char *image,
		     int width,int height);


/* copy image src to image dst, with blend factor and
 * conserving transparency */
void copyImlibImage(ImlibData *id,
		    ImlibImage *dst,ImlibImage *src,
		    int blend);


/* copy image src to image dst at x,y without blend factor
 */
void putImlibImage(ImlibData *id,
		   ImlibImage *dst,ImlibImage *src,
		   int x,int y);

/* print a string on an image
 */
void printStringImage(int x,int y,ImlibImage *im,char *str,
		      FontInfo *font,
		      int *gradCol);

#endif
