#include <X11/Xlib.h>
#include <X11/Xresource.h>
#include <X11/Xutil.h>

#include "debug.h"
#include "utils.h"
#include "gadget.h"
#include "font.h"
#include "tooltips.h"
#include "pixmaps.h"
#include "Xcore.h"

Window tooltip_win=0;
ImlibImage **tooltip_icons=NULL;
int nb_tooltip_icons=0;

void free_tooltip() {
  int i;
  if (tooltip_win)
    XDestroyWindow(display,tooltip_win);

  assert((tooltip_icons==NULL)==(nb_tooltip_icons==0),"Inconsistent data");
  for(i=0;i<nb_tooltip_icons;i++)
    if (tooltip_icons[i])
      Imlib_kill_image(im_data,tooltip_icons[i]);

  FREE(tooltip_icons);
  tooltip_icons=NULL;
  nb_tooltip_icons=0;
  tooltip_win=0;
}


static ImlibImage* setGeom_getBg(Gadget *gad,
				 int width,int height,
				 int *xx,int *yy,int keepwin) {

  Window dummy;
  unsigned int rw,rh;
  int x,y;
  XSizeHints          sh;
  ImlibImage *im=NULL;

  XSetWindowAttributes attr;
  Atom                a;
  XClassHint         *xch;
  MWMHints            mwm;
  unsigned long       val;
  Display            *disp;
  Window root,win;

  pushArgs(256,(BUF,"%s,%d,%d,%s,%s",
		PTR2STR(gad),width,height,PTR2STR(xx),PTR2STR(yy)));

  /* only rw,rh are interesting */
  XGetGeometry(display,XDefaultRootWindow(display),
	       &dummy,&x,&x,&rw,&rh,&x,&x);

  if (gad->x<rw/2) { /* right tooltip */
    x=gad->x+gad->width+16;
  } else { /* left tooltip */
    x=gad->x-width-16;
  }
  if (x<0) x=0;
  else if (x+width>rw)
    x=rw-width;
  y=gad->y-gad->height/2;
  if (y<0) y=0;
  else if (y+height>rh)
    y=rh-height;

  disp=display;
  if (!disp) {
    fprintf(stderr, "FancyLauncher Error: display not opened\n");
    exit(1);
  }

  if (!(keepwin && tooltip_win)) {
    hide_tooltip();

    /* create a window with everything set */
    attr.backing_store = NotUseful;
    attr.override_redirect = False;
    attr.colormap = Imlib_get_colormap(im_data);
    attr.border_pixel = 0;
    attr.background_pixel = 0;
    attr.save_under = False;
    attr.event_mask = 0;
    root=DefaultRootWindow(disp);
    tooltip_win = win = XCreateWindow(disp, root,
				      x,y,width,height, 0,
				      im_data->x.depth, InputOutput,
				      Imlib_get_visual(im_data),
				      CWOverrideRedirect | CWSaveUnder |
				      CWBackingStore | CWColormap | CWBackPixel |
				      CWBorderPixel | CWEventMask, &attr);
    XSetTransientForHint(disp, win, root);
    
    /* set hints to be borderless */
    mwm.flags = MWM_HINTS_DECORATIONS;
    mwm.functions = 0;
    mwm.decorations = 0;
    mwm.inputMode = 0;
    mwm.status = 0;
    a = XInternAtom(disp, "_MOTIF_WM_HINTS", False);
    XChangeProperty(disp, win, a, a, 32, PropModeReplace,
		    (unsigned char *)&mwm, sizeof(MWMHints) / 4);
    debugPrint(("borderless set ... "));
    
    /* set the window title , name , class */
    XStoreName(disp, win, "FancyLauncher");
    xch = XAllocClassHint();
    xch->res_name = "FancyLauncher Tooltip";
    xch->res_class = "FancyLauncher";
    XSetClassHint(disp, win, xch);
    XFree(xch);
    debugPrint(("title set ...\n"));
    
    /* set the default layer to above */
    val = 255;
    a = XInternAtom(disp, "_WIN_LAYER", False);
    XChangeProperty(disp, win, a, XA_CARDINAL, 32, PropModeReplace,
		    (unsigned char *)&val, 1);
    
    /* set skip focus, skip winlist dont cover skip taskbar flags */
    val = (1 << 0) | (1 << 1) | (1 << 2) | (1 << 5);
    a = XInternAtom(disp, "_WIN_HINTS", False);
    XChangeProperty(disp, win, a, XA_CARDINAL, 32, PropModeReplace,
		    (unsigned char *)&val, 1);
    
    assert(gad!=NULL,"Null gadget as argument!");
  }

  /* set the size hints */
  sh.flags = PSize | PMinSize | PMaxSize | PPosition;
  sh.x = x;
  sh.y = y;
  sh.width = width;
  sh.height = height;
  sh.min_width = 16;
  sh.min_height = 16;
  sh.max_width = rw;
  sh.max_height = rh;
  XSetWMNormalHints(display, tooltip_win, &sh);
  debugPrint(("Tooltip: %d,%d %dx%d\n",x,y,width,height));

  if (IS_TRANSPARENT(gad->tooltip_bg))
    im=snapshotImage(im_data,x,y,width,height,0);
  else if (IS_PIXMAP(gad->tooltip_bg))
    trace( im=getImage(im_data,CHARS(GET_PIXMAP(gad->tooltip_bg)),width,height) );
  if (im==NULL)
    trace( im=getSolidImage(width,height,im_data,
			    IS_COLOR(gad->tooltip_bg)
			    ? GET_COLOR(gad->tooltip_bg)
			    : 0x707070) );

  *xx=x;
  *yy=y;
  return im;
}


void hide_tooltip() {
  debugPrint(("hide_tooltip()\n"));
  if (tooltip_win) {
    XUnmapWindow(display,tooltip_win);
    XDestroyWindow(display,tooltip_win);
    tooltip_win=0;
    XFlush(display);
  }
}


void show_tooltip(Gadget *gad,char *text,int keepwin) {
  debugPrint(("show_tooltip(%s)\n",text));
  pushMe();

  if (gad) {
    ImlibImage *im;
    int width=0,height=0,x,y;

    if (gad->tooltip_font==NULL)
      gad->tooltip_font=loadFont(NULL,0);
    if (gad->tooltip_font==NULL)
      return;

    switch(text ? -1 : gad->gadget.any.type) {

    case BUTTON_TYPE:
      {
	int i,n,len=0,icon_w=0,text_h=gad->tooltip_font->height;
	int bx,by;

	debugPrint(("Tooltip font: %s %dx%d\n",
		    gad->tooltip_font->name,
		    gad->tooltip_font->width,
		    gad->tooltip_font->height));

	n=NACTIONS(gad);
	for(i=0;i<n;i++)
	  if (BUT(gad).action[2*i+1]) {
	    int l=strlen(CHARS(BUT(gad).action[2*i+1]));
	    if (l>len) len=l;

	    if (i<nb_tooltip_icons &&
		tooltip_icons[i] &&
		tooltip_icons[i]->rgb_height>text_h)
	      height+=tooltip_icons[i]->rgb_height+4;
	    else
	      height+=text_h+4;

	    if (i<nb_tooltip_icons &&
		tooltip_icons[i] &&
		tooltip_icons[i]->rgb_width>icon_w)
	      icon_w=tooltip_icons[i]->rgb_width;

	    debugPrint(("Button %d : %s\n",i+1,CHARS(BUT(gad).action[2*i+1])));
	  }

	if (height==0 || len==0)
	  return;

	bx=gad->tooltip_bw;
	by=gad->tooltip_bh;
	icon_w+=4;
	trace(im=setGeom_getBg(gad,
			       width=len*gad->tooltip_font->width+2*bx+icon_w+1,
			       height=height+2*by+1,
			       &x,&y,keepwin));
	if (!im)
	  return;

	for(i=0;i<n;i++)
	  if (BUT(gad).action[2*i+1]) {
	    int offsy,jumph;

	    jumph=text_h+4;
	    offsy=0;
	    if (i<nb_tooltip_icons &&
		tooltip_icons[i]) {
	      trace(putImlibImage(im_data,im,tooltip_icons[i],bx,by));
	      if (tooltip_icons[i]->rgb_height>text_h) {
		offsy=(tooltip_icons[i]->rgb_height-text_h)/2;
		jumph=tooltip_icons[i]->rgb_height+4;
	      }
	    }

	    trace(printStringImage(bx+icon_w,by+offsy,im,
				   CHARS(BUT(gad).action[2*i+1]),
				   gad->tooltip_font,
				   gad->gradColor));
	    by+=jumph;
	  }

	break;
      }

    case PERIODIC_TYPE:
      text=PERIODIC(gad).tooltipGadget(gad); /* static */

    default:
      if (text) {
	int i,lines,len,col,bx,by,text_h;
	for(i=0,lines=0,len=0,col=0;text[i];i++,col++) {
	  if (text[i]=='\n') {
	    lines++;
	    if (col>len)
	      len=col;
	    col=0;
	  }
	}
	if (col>len)
	  len=col;
	if (col)
	  lines++;
	
	if (lines==0 || len==0)
	  return;
	
	bx=gad->tooltip_bw;
	by=gad->tooltip_bh;
	text_h=gad->tooltip_font->height;
	trace(im=setGeom_getBg(gad,
			       width=len*gad->tooltip_font->width+2*bx+1,
			       height=text_h*lines+2*by+1,
			       &x,&y,keepwin));
	if (!im)
	  return;

	i=0;
	while(text[i]) {
	  trace(printStringImage(bx,by,im,
				 text+i,gad->tooltip_font,
				 gad->gradColor));
	  while(text[i]) {
	    char c=text[i++];
	    if (c=='\n')
	      break;
	  }
	  by+=text_h;
	}

      } else
	return;
   
    }

    Imlib_apply_image(im_data,im,tooltip_win);
    XMapWindow(display,tooltip_win);
    XMoveResizeWindow(display,tooltip_win,x,y,width,height);
    Imlib_kill_image(im_data,im);
    XFlush(display);
    debugPrint(("Tooltip at %d,%d %dx%d\n",x,y,width,height));
  }
}

